package Tests::process;

use Tests::Base;
@ISA = qw( Tests::Base );

sub execute
{
	my $self = shift;
	return( $self -> probe_return( undef, 
			$self -> opt( 'proc' ) || 'init',
			$self -> opt( 'min' ) || '0',
			$self -> opt( 'max' ) || '9999' )
	);
}

1;

__END__

=head1 NAME

Tests::process - checks that certain processes are(n't) running, requires probe

=head1 SYNOPSIS

	<process proc="process" min="minimum" max="maximum"/>

=head1 DESCRIPTION

Verifies that certain processes are(n't) running on the host.  Creative use of the
min/max values can allow testing for everything from something that is running, 
something out of control, or things that shouldn't be running.

Common values might be:

	<process proc="syslogd" min="1" max="1"/>

	Verify syslog is running

	<process proc="httpd" min="3" max="50"/>

	Verify you aren't being slashdotted.  :)

	<process proc="t0rn" min="0" max="0"/>

	Verify you haven't got a rootkit installed

The probe should check for processes in a way that can't be trojaned, i.e. under linux
it will actually spider the /proc filesystem, instead of using 'ps'.

=head2 proc

The name of the process to check.  The actal value depends on the probe for the OS,
for example under Linux the proc value should be the value of /proc/?/status in the
Name: field, where ? is the processid.

=head2 min

The minimum number allowed to be running.  May be zero if it doesn't have to be running.

=head2 max

The maximum number of this process allowed to be running.  May also be zero, to alert
if something is running that shouldn't, i.e. t0rn kits or a shell server.

=head1 SEE ALSO

sentinel, probe, librarian, sentinel.conf

=head1 AUTHOR

Steve Slaven - http://hoopajoo.net
